package net.pterodactylus.sone.template;

import java.util.Map;

import javax.annotation.Nullable;

import net.pterodactylus.sone.data.IdBuilder;
import net.pterodactylus.sone.data.Sone;
import net.pterodactylus.util.template.Filter;
import net.pterodactylus.util.template.TemplateContext;

import com.google.common.base.Function;
import com.google.common.base.Optional;

/**
 * Filter that {@link IdBuilder builds IDs} from a piped-in element ID and a Sone or Sone ID given as parameter “sone.”
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class BuildIdFilter implements Filter {

	private final IdBuilder idBuilder = new IdBuilder();

	@Override
	public Object format(TemplateContext templateContext, Object data, Map<String, Object> parameters) {
		Optional<String> soneId = getSoneId(parameters);
		if (!soneId.isPresent()) {
			return null;
		}
		Optional<String> elementId = Optional.fromNullable(data).transform(getStringValue());
		if (!elementId.isPresent()) {
			return null;
		}
		return idBuilder.buildId(soneId.get(), elementId.get());
	}

	private Optional<String> getSoneId(Map<String, Object> parameters) {
		Object soneObject = parameters.get("sone");
		if (soneObject instanceof String) {
			return Optional.of((String) soneObject);
		} else if (soneObject instanceof Sone) {
			return Optional.of(((Sone) soneObject).getId());
		}
		return Optional.absent();
	}

	private Function<? super Object, String> getStringValue() {
		return new Function<Object, String>() {
			@Nullable
			@Override
			public String apply(Object input) {
				return (input != null) ? input.toString() : null;
			}
		};
	}

}
